# Generated by Django 3.1.14 on 2022-05-06 16:49

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('dubletten_tool', '0006_statusbuttongroup_statusbuttongrouptype_statusbuttonproxy_statusbuttonproxytype'),
    ]

    operations = [
        migrations.AlterField(
            model_name='statusbuttongroup',
            name='related_instance',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='dubletten_tool.group'),
        ),
        migrations.AlterField(
            model_name='statusbuttonproxy',
            name='related_instance',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='dubletten_tool.personproxy'),
        ),
    ]



# Generated by Django 3.1.14 on 2022-05-10 07:23

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dubletten_tool', '0010_auto_20220510_0611'),
    ]

    operations = [
        migrations.AlterField(
            model_name='group',
            name='note',
            field=models.TextField(blank=True, max_length=2000, null=True),
        ),
        migrations.AlterField(
            model_name='personproxy',
            name='note',
            field=models.TextField(blank=True, max_length=2000, null=True),
        ),
        migrations.DeleteModel(
            name='Note',
        ),
    ]



import Levenshtein as lev
from .models import PersonProxy, Group, Suggestions

singles = PersonProxy.objects.filter(status="single")

def get_levensthein(person, distance=2):
    print("levensthein running")
    name = person.name
    first = person.first_name
    persons = [(p.person.name, p.person.first_name, p.person.id, p) for p in singles]
    res = []
    for n, f, pk, p in persons:
        dist_n = lev.distance(name, n)
        dist_f = lev.distance(first, f)

        if dist_n == 0:
            if 0 < dist_f <= distance:
                #res.append((n,f, pk))
                res.append(p)


        elif dist_f == 0:
            if 0 < dist_n <= distance:
                #res.append((n,f, pk))
                res.append(p)

    return res


def check_all_names(obj):
    """
    Get Levensthein distance for obj against all other groups and singles. 
    Obj: Group or a PersonProxy instance.
    """
    print("all names running")
    if isinstance(obj, Group):
        names = obj.all_names
        first_names = obj.all_first_names
    
    elif isinstance(obj, PersonProxy):
        names = obj.names
        first_names = obj.first_names

    res_g = []
    res_s = []

    for s in singles:
        if s.first_names_set.intersection(first_names) and s.names_set.intersection(names):
            res_s.append(s)
    
    # for g in Group.objects.all():
    #     if g.all_names.intersection(names) and g.all_first_names.intersection(first_names):
    #         res_g.append(g)
    
    return res_g, res_s

def get_name_suggestions(pp_pk):
    print("get_name_suggestions running")
    suggestions = Suggestions.objects.all()[0].data

    return suggestions[str(pp_pk)]






    





from django.forms import ModelForm, HiddenInput
from django.urls import reverse

from crispy_forms.helper import FormHelper
from crispy_forms.layout import Layout, Button, Submit
from .models import Group, PersonProxy

class GroupForm(ModelForm):
    class Meta: 
        model = Group
        fields = ("note",)

    def __init__(self, inst_id, *args, **kwargs):
        super(GroupForm, self).__init__( *args, **kwargs)

        print("in Group Form, inst_id", inst_id)
        print("args:", args)
        print("kwargs:", kwargs)
        #self.fields['id'].widget = HiddenInput()
        #self.fields['id'].initial = inst_id

        self.helper = FormHelper()
        self.helper.form_class = "NoteForm"
        self.helper.form_tag = True
        self.helper.help_text_inline = True
        self.helper.form_action = reverse("dubletten_tool:handle_note_form", kwargs={"g_id":inst_id, "type":"group"})
        self.helper.add_input(Submit("submit", "save", css_id=f"post_note_{inst_id}"))
        self.helper.add_input(Button("cancel", "Cancel", css_class="btn-danger", onclick=f"abortNoteEdit({inst_id},'group')"))
        print(self.fields)
        #self.helper.layout = Layout("id", "note")



class PersonProxyForm(ModelForm):
    class Meta: 
        model = PersonProxy
        fields = ["note"]

    def __init__(self, inst_id, *args, **kwargs):
        super(PersonProxyForm, self).__init__(*args, **kwargs)
        self.helper = FormHelper()
        self.helper.form_class = "NoteForm"
        self.helper.form_tag = True
        self.helper.help_text_inline = True
        self.helper.form_action = reverse("dubletten_tool:handle_note_form", kwargs={"g_id":inst_id, "type":"personproxy"})
        self.helper.add_input(Submit("submit", "save", css_id=f"post_note_{inst_id}"))
        self.helper.add_input(Button("cancel", "Cancel", css_class="btn-danger", onclick=f"abortNoteEdit({inst_id},'personproxy')"))

    


    


from apis_core.apis_labels.models import LabelType

nach_alt = LabelType.objects.filter(name__in=["alternative name", "alternativer Nachname"])
nach_ver = LabelType.objects.get(name="Nachname verheiratet")
nach_ver_alt = LabelType.objects.filter(name__in=["Nachname alternativ verheiratet", "Nachname alternativ vergeiratet"])
nach = []+list(nach_alt)+ list(nach_ver_alt)
nach.appen(nach_ver)
vor = LabelType.objects.get(name="alternativer Vorname")

nach = ["alternative name", "alternativer Nachname", "Nachname verheiratet", "Nachname alternativ verheiratet", "Nachname alternativ vergeiratet"]


from .models import Group, PersonProxy
from apis_core.apis_entities.models import Person
from apis_core.apis_metainfo.models import Collection
import pandas as pd

HSV = Collection.objects.get(name="Import HSV full 22-6-21")
HZAB = Collection.objects.get(name="Import HZAB full 10-3-21")
ACC = Collection.objects.get(name="Import ACCESS full 13-10-21")
MAN = Collection.objects.get(name="manually created entity")

persons = Person.objects.exclude(collection__in=[ACC])


df_per = pd.DataFrame(persons.values()).set_index("id")
per = df_per[["name", "first_name", "gender", "start_date", "end_date", "start_date_written", "end_date_written"]]
per["fullname"] = per.name + ", " + per.first_name


singles = []
def get_groups():
    names_gender = []
    df_test = []
    frames = []
    c = 0
    for d, group in per.groupby(["fullname", "gender"]):
        if len(group) > 1:
            #log.info(f"{len(group)}, {d}, {[f for f in group.index]}")
            names_gender.append((len(group), d, group.index.to_list()))
            #df_test.append({"count":len(group), "key":d})
            c += len(group)
            #frames.append(group)
        elif len(group) == 1:
            singles.append(group.index.to_list()[0])
    
    return names_gender


ng = get_groups()



for s in singles:
    per = Person.objects.get(id=s)
    prox = PersonProxy.objects.get(person=per, status="single")
    prox.save()





# Generated by Django 3.1.14 on 2022-04-30 01:48

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('dubletten_tool', '0003_auto_20220430_0140'),
    ]

    operations = [
        migrations.RenameField(
            model_name='personproxy',
            old_name='first_names',
            new_name='_first_names',
        ),
        migrations.RenameField(
            model_name='personproxy',
            old_name='names',
            new_name='_names',
        ),
    ]



# Generated by Django 3.1.14 on 2022-04-30 01:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dubletten_tool', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='group',
            options={'ordering': ['name']},
        ),
        migrations.AlterModelOptions(
            name='personproxy',
            options={'ordering': ['person__name']},
        ),
        migrations.AddField(
            model_name='personproxy',
            name='_names',
            field=models.JSONField(blank=True, null=True),
        ),
    ]



# Generated by Django 3.1.14 on 2022-05-09 09:04

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('dubletten_tool', '0008_group__gender'),
    ]

    operations = [
        migrations.CreateModel(
            name='Note',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('text', models.TextField(blank=True, max_length=2000, null=True)),
            ],
        ),
        migrations.AddField(
            model_name='group',
            name='marked',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='personproxy',
            name='marked',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='group',
            name='note',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='dubletten_tool.note'),
        ),
        migrations.AddField(
            model_name='personproxy',
            name='notes',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='dubletten_tool.note'),
        ),
    ]



# Generated by Django 3.1.14 on 2022-05-09 08:32

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dubletten_tool', '0007_auto_20220506_1649'),
    ]

    operations = [
        migrations.AddField(
            model_name='group',
            name='_gender',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
    ]



# Generated by Django 3.1.14 on 2022-04-30 01:40

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dubletten_tool', '0002_auto_20220430_0131'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='personproxy',
            name='_names',
        ),
        migrations.AddField(
            model_name='personproxy',
            name='first_names',
            field=models.JSONField(null=True),
        ),
        migrations.AddField(
            model_name='personproxy',
            name='names',
            field=models.JSONField(null=True),
        ),
    ]



from django.urls import path
from django.conf.urls import url
from .views import get_all_notes, getToolPage, getNoteJSON, HandleNoteForm, get_group_ajax, rename_group, update_group_checkboxes, create_new_group, merge_groups, remove_member, get_singles, get_groups, get_single_ajax, get_person_suggestions
from .detail_view import OverwrittenGenericEntitiesDetailView
app_name = "dubletten_tool"

urlpatterns = [
    path("start/", getToolPage.as_view(), name="tool_page"),
    path("create_group/", create_new_group, name="create_new_group"),
    url(r"^handle_note_form/(?P<g_id>[0-9]+)/(?P<type>[a-z]+)/$", HandleNoteForm.as_view(), name="handle_note_form"),
    path("get_singles/", get_singles, name="get_singles"),
    path("get_all_notes/", get_all_notes, name="get_all_notes"),
    url(r"^get_single_ajax/(?P<s_id>[0-9]+)/$", get_single_ajax, name="get_single_ajax"),
    #url(r"^get_singles/(?P<val>[a-zA-Zäöüß#\-\_()]+)/(?P<type>[a-zA-Zäöüß#\_\-]+)/$", get_singles, name="get_singles"),
    url(r"^get_singles/(?P<val_name>[a-zA-Zäöüß#\-\_()]+)/(?P<val_first>[a-zA-Zäöüß#\_\-]+)/(?P<gender>[A-Za-z]+)/$", get_singles, name="get_singles"),
    url(r"^get_groups/(?P<val>[a-zA-Zäüöß_,\[\]\(\).]+)/(?P<gender>[A-Za-z]+)/$", get_groups, name="get_groups"),
    url(r"get_note_json/(?P<inst_id>[0-9]+)/(?P<type>[a-z]+)/$", getNoteJSON, name="get_note_json"),
    path("merge_groups/", merge_groups, name="merge_groups"),
    path("rename_group/", rename_group, name="rename_group"),
    url(r"^update_group_checkboxes/(?P<btn_id>[0-9]+)/(?P<group_id>[0-9]+)/$", update_group_checkboxes, name="update_group_checkboxes"),
    url(r"^get_group/(?P<g_id>[0-9]+)/$", get_group_ajax, name="get_group_ajax"),
    url(r"^remove_member/(?P<group_id>[0-9]+)/(?P<per_id>[0-9]+)/$", remove_member, name="remove_member"),
    url(r"^get_person_detail/(?P<per_id>[0-9]+)/$", OverwrittenGenericEntitiesDetailView.as_view(), name="get_person_detail"),
        url(r"^get_person_suggestions/(?P<per_id>[0-9]+)/$", get_person_suggestions, name="get_person_suggestions"),





]

update_group_checkboxes


# Generated by Django 3.1.14 on 2022-05-12 08:32

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('apis_entities', '0004_auto_20200722_1231'),
    ]

    operations = [
        migrations.CreateModel(
            name='Group',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(blank=True, max_length=600)),
                ('status', models.CharField(choices=[('unchecked', 'unchecked'), ('checked group', 'checked group'), ('checked for other groups', 'checked for other groups'), ('checked all members', 'checked all members'), ('ready to merge', 'ready to merge'), ('merged', 'merged')], default='unchecked', max_length=300)),
                ('_gender', models.CharField(blank=True, max_length=255, null=True)),
                ('marked', models.BooleanField(default=False)),
                ('note', models.TextField(blank=True, max_length=2000, null=True)),
            ],
            options={
                'ordering': ['name'],
            },
        ),
        migrations.CreateModel(
            name='PersonProxy',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('status', models.CharField(choices=[('candidate', 'Candidate'), ('single', 'Single'), ('merged', 'Merged')], default='single', max_length=300)),
                ('marked', models.BooleanField(default=False)),
                ('note', models.TextField(blank=True, max_length=2000, null=True)),
                ('_names', models.JSONField(null=True)),
                ('_first_names', models.JSONField(null=True)),
                ('person', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='apis_entities.person')),
            ],
            options={
                'ordering': ['person__name'],
            },
        ),
        migrations.CreateModel(
            name='StatusButtonGroupType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=600)),
                ('short', models.CharField(default='BT', max_length=4)),
            ],
        ),
        migrations.CreateModel(
            name='StatusButtonProxyType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=600)),
            ],
        ),
        migrations.CreateModel(
            name='Suggestions',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('data', models.JSONField(blank=True, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='StatusButtonProxy',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.BooleanField(default=False)),
                ('kind', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='dubletten_tool.statusbuttonproxytype')),
                ('related_instance', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='dubletten_tool.personproxy')),
            ],
        ),
        migrations.CreateModel(
            name='StatusButtonGroup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.BooleanField(default=False)),
                ('kind', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='dubletten_tool.statusbuttongrouptype')),
                ('related_instance', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='dubletten_tool.group')),
            ],
        ),
        migrations.AddField(
            model_name='group',
            name='members',
            field=models.ManyToManyField(blank=True, to='dubletten_tool.PersonProxy'),
        ),
    ]



import os
import pandas as pd
from copy import deepcopy
from collections import Counter, defaultdict
import numpy as np 
import re
from apis_core.apis_entities.models import Person, Institution, Place, Event, Work
from apis_core.apis_relations.models import *
from apis_core.apis_metainfo.models import Collection
from apis_core.apis_vocabularies.models import *
from .models import PersonProxy, Group
#import Levenshtein as lev
import logging


log = logging.getLogger("mylogger")
log.setLevel(logging.INFO)

log.info("logger init")

# # base entities
# P = Person
# I = Institution
# PL = Place
# W = Work
# E = Event

# #functions
# F = PersonInstitutionRelation

# #other relations
# PI = PersonInstitution
# PE = PersonEvent
# PP = PersonPlace
# IP = InstitutionPlace

# #collections
# HSV = Collection.objects.get(name="Import HSV full 22-6-21")
# HZAB = Collection.objects.get(name="Import HZAB full 10-3-21")
# ACC = Collection.objects.get(name="Import ACCESS full 13-10-21")
# MAN = Collection.objects.get(name="manually created entity")

# #other
# ENTS = [P, I, PL, W, E]

# def get_hs(instname):
#     if "(" in instname:
#         temp = instname.replace("(", "$", 1)
#         hs = temp.split("$")[1][:-1]
#         return hs
#     else:
#         return None


df_per = pd.DataFrame(Person.objects.values("id","name", "first_name", "gender", "start_date", "end_date", "start_date_written", "end_date_written")).set_index("id")
per = df_per[["name", "first_name", "gender", "start_date", "end_date", "start_date_written", "end_date_written"]]
#per["fullname"] = per.name + ", " + per.first_name
per["sd"] = [str(d) for d in per["start_date"]]
per["ed"] = [str(d) for d in per["end_date"]]


for d, group in per.groupby(["name", "first_name", "gender", "sd", "ed"]):
            bd, dd = d[3], d[4]
            bd = bd.replace("None", "")
            dd = dd.replace("None", "")
            if len(group) > 1:
                if bd and dd:
                    name = (f"{d[0]}, {d[1]} [B+D]")
                elif bd:
                    name = (f"{d[0]}, {d[1]} [B]")
                elif dd:
                    name = (f"{d[0]}, {d[1]} [D]")
                else:
                    name = (f"{d[0]}, {d[1]}")

                g, c = Group.objects.get_or_create(name=name)
                index = group.index.to_list()
                for ix in index:
                    gm, c = PersonProxy.objects.get_or_create(person=Person.objects.get(id=ix))
                    gm.status="candidate"
                    gm.save()
                    g.members.add(gm)
                
            elif len(group) == 1:
                s = group.index.to_list()[0]
                per = Person.objects.get(id=s)
                prox, c = PersonProxy.objects.get_or_create(person=per)
                prox.status = "single"
                prox.save()



# for d, group in per.groupby(["name", "first_name", "gender", "sd", "ed"]):
#     bd, dd = d[3], d[4]
#     bd = bd.replace("None", "")
#     dd = dd.replace("None", "")


# def run():
#     df_per = pd.DataFrame(Person.objects.all().values()).set_index("id")
#     per = df_per[["name", "first_name", "gender", "start_date", "end_date", "start_date_written", "end_date_written"]]
#     #per["fullname"] = per.name + ", " + per.first_name
#     singles = []
#     def get_groups():
#         names_gender = []
#         df_test = []
#         frames = []
#         c = 0
#         for d, group in per.groupby(["name", "first_name", "gender"]):
#             if len(group) > 1:
#                 #log.info(f"{len(group)}, {d}, {[f for f in group.index]}")
#                 names_gender.append((len(group), d, group.index.to_list()))
#                 #df_test.append({"count":len(group), "key":d})
#                 c += len(group)
#                 #frames.append(group)
#             elif len(group) == 1:
#                 singles.append(group.index.to_list()[0])
#         return names_gender


#     ng = get_groups()


#     def create_groups():   
#         for count, el in enumerate(ng):
#             group, c = Group.objects.get_or_create(name=f"{el[1][0]} [{el[1][1]}]")
#             print(f"{count}/{len(ng)}")
#             for index in el[2]:
#                 gm, c = PersonProxy.objects.get_or_create(person=Person.objects.get(id=index))
#                 group.members.add(gm)
#                 gm.status="candidate"

#     create_groups()

#     for s in singles:
#         per = Person.objects.get(id=s)
#         prox = PersonProxy.objects.create(person=per, status="single")
#         prox.save()

#run()

###### Levenshtein Distance #####

# DISTANCE_VALUE = 2
# def lev_distance(a, series, container):
#     for pk, b in series.items():
#         dist = lev.distance(a,b)
#         if 0 < dist <= DISTANCE_VALUE:
#             container.append((dist, b, pk))
#     return container

# test = "Müller"
# container = []

# #container = lev_distance(test, per.name, container)







# Generated by Django 3.1.14 on 2022-04-30 18:37

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('dubletten_tool', '0004_auto_20220430_0148'),
    ]

    operations = [
        migrations.CreateModel(
            name='Suggestions',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('data', models.JSONField(blank=True, null=True)),
            ],
        ),
    ]



# Generated by Django 3.1.14 on 2022-05-06 10:58

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('dubletten_tool', '0005_suggestions'),
    ]

    operations = [
        migrations.CreateModel(
            name='StatusButtonGroupType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=600)),
            ],
        ),
        migrations.CreateModel(
            name='StatusButtonProxyType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=600)),
            ],
        ),
        migrations.CreateModel(
            name='StatusButtonProxy',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.BooleanField(default=False)),
                ('kind', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='dubletten_tool.statusbuttonproxytype')),
                ('related_instance', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='dubletten_tool.personproxy')),
            ],
        ),
        migrations.CreateModel(
            name='StatusButtonGroup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('value', models.BooleanField(default=False)),
                ('kind', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='dubletten_tool.statusbuttongrouptype')),
                ('related_instance', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to='dubletten_tool.group')),
            ],
        ),
    ]



from django.db import models
from apis_core.apis_entities.models import Person


# class Note(models.Model):
#     text = models.TextField(max_length=2000, null=True, blank=True)

class PersonProxy(models.Model):

    class Meta:
        ordering = ['person__name']
    # todo: __gpirgie__ rename to Deduplication Proxy or something like that
    status_choices = [
        ("candidate", "Candidate"),
        ("single", "Single"),
        ("merged", "Merged"),
    ]
    person = models.OneToOneField(Person, null=False, blank=False, on_delete=models.CASCADE)
    status = models.CharField(max_length=300, choices=status_choices, default="single")
    marked = models.BooleanField(default=False)
    note = models.TextField(max_length=2000, null=True, blank=True)

    _names = models.JSONField(null=True)
    _first_names = models.JSONField(null=True)


    @property
    def alt_names(self):
        nach = ["alternative name", "alternativer Nachname", "Nachname verheiratet", "Nachname alternativ verheiratet", "Nachname alternativ vergeiratet"]
        return [l[0].strip() for l in self.person.label_set.filter(label_type__name__in=nach).values_list("label")]

    @property
    def name_verheiratet(self):
        ver = ["Nachname verheiratet", "Nachname alternativ verheiratet", "Nachname alternativ vergeiratet"]
        return [l[0].strip() for l in self.person.label_set.filter(label_type__name__in=ver).values_list("label")]

    @property 
    def alt_first_names(self):
        return [l[0].strip() for l in self.person.label_set.filter(label_type__name="alternativer Vorname").values_list("label")]

    @property
    def names_list(self):
        name = self.person.name
        alt_names = self.alt_names
        alt_names.append(name)
        return alt_names

    @property
    def first_names_list(self):
        first_name = self.person.first_name
        alt_first_names = self.alt_first_names
        alt_first_names.append(first_name)
        return alt_first_names

    @property
    def names_set(self):
        return set(self.names_list)

    @property
    def first_names_set(self):
        return set(self.first_names_list)

    @property
    def names(self):
        return set(self._names)

    @property
    def first_names(self):
        return set(self._first_names)

    @property
    def name(self):
        return f"{self.person.name}, {self.person.first_name}"



class Group(models.Model):
    """Group: A collection of Person-Instances storing possible duplicates. 
    Each group should run through several filter, split and add processes to finally be merged into one entity. 
    A Group holds GroupingCandidates, which serve as proxies for the actual Person-instances.

    Args:
        models (_type_): 
        
    Fields:
    """

    class Meta:
        ordering = ["name"]

    status_choices_group = [
        ("unchecked", "unchecked"),
        ("checked group", "checked group"),
        ("checked for other groups", "checked for other groups"),
        ("checked all members", "checked all members"),
        ("ready to merge", "ready to merge"),
        ("merged", "merged")
    ]
    name = models.CharField(max_length=600, blank=True)
    members = models.ManyToManyField(PersonProxy, blank=True)
    status = models.CharField(max_length=300, choices=status_choices_group, default="unchecked")
    _gender = models.CharField(max_length=255, null=True, blank=True)
    marked = models.BooleanField(default=False)
    note = models.TextField(max_length=2000, null=True, blank=True)

    @property
    def all_notes(self):
        notes = [{"person_id":el.person.id, "note":el.note} for el in self.members.all()]
        return notes

    @property
    def gender(self):
        return self._gender

    @gender.setter
    def gender(self, value):
        self._gender = value

    @property
    def buttons(self):
        return self.statusbuttongroup_set.all()

    @property
    def count(self):
        return self.members.all().count()

    @property
    def all_names(self):
        names = set()
        for m in self.members.all():
            names = names.union(m.names)
        
        return names

    @property
    def all_first_names(self):
        first_names = set()
        for m in self.members.all():
            first_names = first_names.union(m.first_names)
        
        return first_names


class Suggestions(models.Model):
    data = models.JSONField(null=True, blank=True)


class StatusButtonGroupType(models.Model):
    name = models.CharField(max_length=600, null=False, blank=False)
    short = models.CharField(max_length=4, null=False, blank=False, default="BT")

    def add_to_all_groups(self):
        all_groups = Group.objects.all()
        for g in all_groups:
            sb, c = StatusButtonGroup.objects.get_or_create(kind=self, related_instance=g)
            sb.save()

class StatusButtonGroup(models.Model):
    kind = models.ForeignKey(StatusButtonGroupType, on_delete=models.CASCADE)
    related_instance = models.ForeignKey(Group, on_delete=models.CASCADE)
    value = models.BooleanField(default=False)

    def toggle_status(self):
        self.value = not self.value
        self.save()

class StatusButtonProxyType(models.Model):
    name = models.CharField(max_length=600, null=False, blank=False)

    def add_to_all_groups(self):
        all_proxies = PersonProxy.objects.all()
        for p in all_proxies:
            sb, c = StatusButtonProxy.objects.get_or_create(kind=self, related_instance=p)
            sb.save()

class StatusButtonProxy(models.Model):
    kind = models.ForeignKey(StatusButtonProxyType, on_delete=models.CASCADE)
    related_instance = models.ForeignKey(PersonProxy, on_delete=models.CASCADE) 
    value = models.BooleanField(default=False)







#     removed = models.ManyToManyField(GroupingCandidate, blank=True, null=True)
#     added = models.ManyToManyField(GroupingCandidate, blank=True, null=True)




    







from django.shortcuts import render
from django.http import HttpResponse, JsonResponse
from django.views.generic import View, TemplateView
from django.utils.decorators import method_decorator
# Create your views here.
from django.template.loader import render_to_string
from django.contrib.auth.decorators import login_required, user_passes_test
#from .grouping_script_1 import run
from .models import Group, PersonProxy, StatusButtonGroup, StatusButtonProxy, StatusButtonGroupType
import json
from .suggestions import get_levensthein, check_all_names, get_name_suggestions
#run()
from .forms import GroupForm, PersonProxyForm


def create_new_buttons(group):
         
    for btn in StatusButtonGroupType.objects.all():
        b, c = StatusButtonGroup.objects.get_or_create(kind=btn, related_instance=group)
        b.save()

    group.save()

@method_decorator(login_required, name="dispatch")
class getToolPage(TemplateView):
    template_name = "tool_page.html"
    print("getToolPage called")

    def get_context_data(self, **kwargs):
        #ng = get_groups()
        groups = Group.objects.all()
        #singles = PersonProxy.objects.filter(status="single")
        context = {}

        context["groups"] = groups
        #context["singles"] = singles
        return context


def get_person_suggestions(request, **kwargs):

    if request.method == "GET":
        per_id = kwargs.get("per_id")
        pp = PersonProxy.objects.get(person__id=per_id)
        if pp.status == "candidate":
            pgroup = pp.group_set.all()[0]
        else:
            pgroup = None
        template = "person_suggestions.html"
        lev_data = get_levensthein(pp.person)
        l_singles = []
        l_groups = []
        for el in lev_data:
            if el.status == "single":
                l_singles.append(el)
            elif el.status == "candidate":
                group = el.group_set.all()[0]
                l_groups.append(group)
            else:
                continue
        levs = {"groups": list(set(l_groups)), "singles": list(set(l_singles))}

        #all_names = check_all_names(pp)
        names = get_name_suggestions(pp.id)
        all_names = PersonProxy.objects.filter(id__in=names)
        names_singles = all_names.filter(status="single")
        print(names_singles)
        in_group = all_names.filter(status="candidate")
        names_groups = {g for p in in_group for g in p.group_set.all()} 
        if pgroup:
            names_groups.discard(pgroup)          
        context = {
            "instance": pp,
            "levs": levs,
            "names": {"groups": list(names_groups), "singles":names_singles}
        }
        html = render_to_string(template, context, request)

        return JsonResponse({"html":html})

def get_group_suggestions(request, **kwargs):
    """
    TODO: Implement this for groups. 
    Ignore Levensthein, but accumulate all names suggestions 
    for all group members, ignoring the calling group.
    """
    pass

def rename_group(request):
    if request.method == "POST":
        print(request.POST)
        data = json.loads(request.POST.get("data"))
        new_name = data.get("new_name")
        if not new_name or new_name in [None, "Null", "none", "None", "null"]:
            new_name = "Unnamed Group"
        g_id = data.get("g_id")
        group = Group.objects.get(id=g_id)
        old_name = Group.name
        group.name = new_name 
        group.save()
        msg = f"Renamed Group {g_id} from {old_name} to {new_name}."

        return JsonResponse({"data":{"group_id": g_id, "msg":msg, "count":group.count, "new_name":group.name}})

class HandleNoteForm(View):

    # todo: rename everything related in here from group to note (noteForm, inst_id instead of g-id, etc)

    def get(self, *args, **kwargs):
        template = "note_form.html"
       
        g_id = self.kwargs.get("g_id")
        type = self.kwargs.get("type")
        print("get method of handlegroupfrom called, id and type is: ", g_id, type)


        if type == "group":
            instance = Group.objects.get(id=g_id)
            form = GroupForm(g_id, instance=instance)

        else: 
            instance = PersonProxy.objects.get(person__id=g_id)
            form = PersonProxyForm(g_id, instance=instance)

        html = render_to_string(template, {"form":form, "instance":instance}, self.request)

        return JsonResponse({"html": html, "success":True})

    def post(self, *args, **kwargs):
        g_id = self.kwargs.get("g_id")
        type = self.kwargs.get("type")
        print("post method of handlegroupfrom called, id and type is: ", g_id, type)


        if type == "group":
            instance = Group.objects.get(id=g_id)
            form = GroupForm(g_id, self.request.POST, instance=instance)

        else:
            instance = PersonProxy.objects.get(person__id=g_id)
            form = PersonProxyForm(g_id, self.request.POST, instance=instance)
            

        success = False
        if form.is_valid():
            ent = form.save()
            ent.save()

            print("SAVED NOTE")

            success = True
            res_msg = "Saved Note!"
        else:
            res_msg = "Form was invalid"

        return JsonResponse({"msg": res_msg, "success":success, "inst_id":g_id, "type":type})

def get_all_notes(request):
    template = "all_notes.html"
    singles = [{"name":s.name, "id":s.person.id, "note":s.note} for s in PersonProxy.objects.filter(status="single") if s.note]
    candidates = [s.group_set.all()[0] for s in PersonProxy.objects.filter(status="candidate") if s.note]
    groups = [g for g in Group.objects.all() if g.note] + candidates

    groups = set(groups)

    groups = [{"name": g.name, "id":g.id, "note":g.note} for g in groups]
    context = {"groups": groups, "members":candidates, "singles":singles}

    html = render_to_string(template, context, request) 

    return JsonResponse({"html":html})



def get_groups(request, **kwargs):

    if request.method == "GET":
        filter = kwargs.get("val")
        filter = filter.replace("__", " ")
        print("filter was: ", filter)
        gender = kwargs.get("gender")
        print("gender arrived as: ", gender)
        if gender == "Other":
            gender = [None, "third gender"]
        else: 
            gender = [gender.lower()]
            
        print(gender)
        context = {}
        if not filter:
            context["groups"] = Group.objects.filter(_gender__in=gender)

        else: 
            groups = Group.objects.filter(name__istartswith=filter, _gender__in=gender)
            context["groups"] = groups

        count = len(context["groups"])
        context["group_count"] = count

        html = render_to_string("group_list.html", context, request)

        return JsonResponse({"html": html, "group_count": count})


def renderNoteHtml(instance, type, request):
    template = "note_template.html"

    if instance.note:
        note = instance.note
    else: 
        note = ""

    if type == "group":
        name = instance.name
    else: 
        name = f"{instance.person.name}, {instance.person.first_name}"
        instance = instance.person

    context = {"instance": instance, "note":note, "name":name, "type":type}

    html = render_to_string(template, context, request)

    return html



def getNoteJSON(request, **kwargs):

    if request.method == "GET":
        pk = kwargs.get("inst_id")
        type = kwargs.get("type")
        print("in getNoteJson, pk, type is: ", pk, type)
        if type == "group":
            instance = Group.objects.get(id=pk)
        else: 
            instance = PersonProxy.objects.get(person__id=pk)
        
        html = renderNoteHtml(instance, type, request)

        return JsonResponse({"html":html, "success":True})

def get_singles(request, **kwargs):
    
    if request.method == "GET":
        #singles = {s.id: {"name":s.person.name, "first_name": s.person.first_name,"start": s.person.start_date,"end":s.person.end_date} for s in PersonProxy.objects.filter(status="single")}
        
        # text = kwargs.get("val")
        # type = kwargs.get("type")
        # print("text arrived as: ", text)
        
        # if not text:
        #     singles = [(s.person.id, s.person.name, s.person.first_name, s.person.start_date, s.person.end_date) for s in PersonProxy.objects.filter(status="single")]
        # else:
        #     print("used filtered singles")
            
        #     text = text.replace("__", " ")

        #     if type == "name":
        #         singles = [(s.person.id, s.person.name, s.person.first_name, s.person.start_date, s.person.end_date) for s in PersonProxy.objects.filter(person__name__istartswith=text).filter(status="single")]
        #     elif type == "first_name":
        #         singles = [(s.person.id, s.person.name, s.person.first_name, s.person.start_date, s.person.end_date) for s in PersonProxy.objects.filter(person__first_name__istartswith=text).filter(status="single")]

        #     print("filtered singles len: ", len(singles))

        text_name = kwargs.get("val_name")
        text_first = kwargs.get("val_first")
        print(f"Arrived as - name: {text_name}, first: {text_first}")

        gender = kwargs.get("gender")
        print("gender arrived as: ", gender)
        if gender == "Other":
            gender = [None, "third gender"]
        else: 
            gender = [gender.lower()]

        if text_name == "false":
            text_name = False
        else: 
            text_name = text_name.replace("__", " ")

        if text_first == "false":
            text_first = False
        else:
            text_first = text_first.replace("__", " ")

        if not text_name and not text_first:
            singles = [(s.person.id, s.person.name, s.person.first_name, s.person.start_date, s.person.end_date) for s in PersonProxy.objects.filter(status="single", person__gender__in=gender)]
        elif not text_first:
            singles = [(s.person.id, s.person.name, s.person.first_name, s.person.start_date, s.person.end_date) for s in PersonProxy.objects.filter(person__name__istartswith=text_name).filter(status="single", person__gender__in=gender)]
        elif not text_name:
            singles = [(s.person.id, s.person.name, s.person.first_name, s.person.start_date, s.person.end_date) for s in PersonProxy.objects.filter(person__first_name__icontains=text_first).filter(status="single", person__gender__in=gender)]
        else:
            singles = [(s.person.id, s.person.name, s.person.first_name, s.person.start_date, s.person.end_date) for s in PersonProxy.objects.filter(person__first_name__icontains=text_first, person__name__istartswith=text_name).filter(status="single", person__gender__in=gender)]

        return JsonResponse({"singles":singles})

def get_group_ajax(request, **kwargs):

    if request.method == "GET":
        g_id = kwargs.get("g_id")
        group = Group.objects.get(id=g_id)
        members = [(v, [r for r in v.person.personinstitution_set.all().order_by("relation_type__name")]) for v in group.members.all()]
        members.sort(key=lambda x: len(x[1]), reverse=False)
        context = {
            "group":group,
            "members": members,
        }

        html = render_to_string("member_list.html", context, request)

        return JsonResponse({"html":html})
        
def get_single_ajax(request, **kwargs):

    if request.method == "GET":
        s_id = kwargs.get("s_id")
        single = PersonProxy.objects.get(person__id=s_id)
        if not single.status == "single":
            print("WAS NOT A SINGLE")
        
        context = {
            "p": single,
            "rels": [r for r in single.person.personinstitution_set.all().order_by("relation_type__name")]
        }

        html = render_to_string("singles_list.html", context, request)
        return JsonResponse({"html":html})

def update_group_checkboxes(request, **kwargs):
    if request.method == "POST":
        template = "group_buttons.html"
        btn_id = kwargs.get("btn_id")
        g_id = kwargs.get("group_id")

        print(btn_id)
        print(g_id)
        group = Group.objects.get(id=g_id)
        btn = StatusButtonGroup.objects.get(id=btn_id)
        btn.toggle_status()
        html = render_to_string(template, {"group":group}, request)
        return JsonResponse({"html":html})


def create_new_group(request):
    if request.method == "POST":
        d = json.loads(request.POST.get("DATA"))


        data = d.get("data")
        name = d.get("new_name")
    
        if not name:
            name = "New Group"

        new_group = Group.objects.create(name=name)
        if name == "New Group":
            new_group.name += " "+str(new_group.id)
            new_group.save()

        updates = {}
        former_singles = []
        for k, v in data.items():
            print(k, v)
            if v:
                if k != "singles":
                    group = Group.objects.get(id=k)
                   
                pers = PersonProxy.objects.filter(person__id__in=v)
                print("pers is", pers)
                for p in pers:
                    if k != "singles":
                        group.members.remove(p)
                        group.save()
                    else: 
                        p.status = "candidate"
                        former_singles.append(p.person.id)
                        p.save()
                    new_group.members.add(p)

                if k != "singles":
                    count = group.count
                    g_id = group.id
                    if count == 0:
                        print("In count == 0 gid, count", g_id, count)
                        group.delete()
                        

                    elif count == 1:
                        print("IN coutn == 1 gid, count", g_id, count)

                        per = group.members.all()[0]
                        per.status = "single"
                        per.save()
                        group.delete()
                        count = 0


                    else:
                        group.save()
               
                    updates.update({g_id:count})
                new_group._gender = new_group.members.all()[0].person.gender
                new_group.save()
        create_new_buttons(new_group)

        print("updates is: ", updates)
        res = {"new_group_id": new_group.id, "new_group": new_group.name, "new_group_count":new_group.count, "former_singles":former_singles,"group_updates":updates}
        
        #todo: add message 
        return JsonResponse({"data":res})




def merge_groups(request):
    if request.method == "POST":
        d = json.loads(request.POST.get("DATA"))
        print(d)
        name = d.get("new_name")
        groups = d.get("groups")
        singles = d.get("singles")

        new_group = Group.objects.create(name=name)
        if groups:
            for g in groups:
                group = Group.objects.get(id=g)
                for m in group.members.all():
                    new_group.members.add(m)
                note = group.note
                if note and new_group.note:
                    new_group.note += f"\n[[ Notes from merged group ({g}):{note}]]"
                elif note:
                    new_group.note = f"[[ Notes from merged group ({g}):{note}]]"

                new_group.save()
                group.delete()

        if singles: 
            for s in singles:
                print("singles was true")
                print("single id was: ", s)
                pp = PersonProxy.objects.get(person__id=s)
                pp.status = "candidate"
                pp.save()
                new_group.members.add(pp)
            new_group.save()
        
        new_group._gender = new_group.members.all()[0].person.gender
        new_group.save()
        create_new_buttons(new_group)
        
        
        # todo : add message 
        return JsonResponse({"remove_groups": groups, "remove_singles":singles, "add": [new_group.id, new_group.name, new_group.count]})
            



def remove_member(request, **kwargs):
    if request.method == "GET":
        group_id = kwargs.get("group_id")
        per_id = kwargs.get("per_id")

        group = Group.objects.get(id=group_id)
        per = PersonProxy.objects.get(person__id=per_id)
        group.members.remove(per)
        per.status = "single"
        group.save()
        per.save()
        updates = {}

        count = group.count
        if count == 0:
            group.delete()
            

        elif count == 1:
            per = group.members.all()[0]
            per.status = "single"
            per.save()
            group.delete()
            count = 0

        else:
            group.save()
        
        updates.update({group_id:count})

        #todo: add message
        return JsonResponse({"data":updates})

            





# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['dubletten_tool', 'dubletten_tool.migrations']

package_data = \
{'': ['*'], 'dubletten_tool': ['static/dubletten_tool/css/*', 'templates/*']}

install_requires = \
['python-Levenshtein>=0.12.2,<0.13.0']

setup_kwargs = {
    'name': 'dubletten-tool',
    'version': '0.1.3',
    'description': 'APIS module for the VieCPro instance to perform manual deduplication of person instances',
    'long_description': None,
    'author': 'Gregor Pirgie',
    'author_email': 'gregor.pirgie@oeaw.ac.at',
    'maintainer': None,
    'maintainer_email': None,
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<3.9',
}


setup(**setup_kwargs)



import logging

def init_logger():
    LOG = logging.getLogger("mylogger")
    format = logging.Formatter("<<<<< LOGGER >>>>> %(module)s >>> %(funcName)s line:%(lineno)d >>> %(message)s")
    handler = logging.StreamHandler()
    handler.setFormatter(format)
    LOG.addHandler(handler)
    LOG.setLevel(logging.WARNING)



# Generated by Django 3.1.14 on 2022-05-10 06:11

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('dubletten_tool', '0009_auto_20220509_0904'),
    ]

    operations = [
        migrations.RenameField(
            model_name='personproxy',
            old_name='notes',
            new_name='note',
        ),
    ]
